@@ -203,6 +203,10 @@ namespace Astroid {
     update_title_dispatcher.connect (
         sigc::mem_fun (this, &MainWindow::on_update_title));
 
+    /* catch window close events */
+    signal_delete_event ().connect (
+        sigc::mem_fun (this, &MainWindow::on_delete_event));
+
     /* register keys {{{ */
     keys.title = "MainWindow";
 
@@ -630,13 +634,20 @@ namespace Astroid {
     /* focus out */
     ungrab_active ();
 
-    for (int c = 0; c < notebook.get_n_pages(); c++) {
-      ((Mode*) notebook.get_nth_page (c))->pre_close ();
+    for (int c = notebook.get_n_pages () -1; c >= 0; c--) {
+      /* ((Mode*) notebook.get_nth_page (c))->pre_close (); */
+      del_mode (c);
     }
 
     close (); // Gtk::Window::close ()
   }
 
+  bool MainWindow::on_delete_event (GdkEventAny *) {
+    if (!in_quit)
+      quit ();
+    return false;
+  }
+
   void MainWindow::on_yes () {
     answer_yes_no (true);
   }
