@@ -167,14 +167,18 @@ namespace Astroid {
       LOG (debug) << "pc: got ack (s: " << ack.success () << ") , focus: " << ack.focus().mid () << ", e: " << ack.focus().element ();
 
       if (!ack.focus().mid ().empty () && ack.focus().element () >= 0) {
-        thread_view->focused_message = *std::find_if (
+        auto it = std::find_if (
             thread_view->mthread->messages.begin (),
             thread_view->mthread->messages.end (),
             [&] (auto &m) { return ack.focus().mid () == m->safe_mid (); });
 
-        thread_view->state[thread_view->focused_message].current_element = ack.focus().element ();
+        if (it != thread_view->mthread->messages.end ()) {
+          thread_view->focused_message = *it;
 
-        thread_view->unread_check ();
+          thread_view->state[thread_view->focused_message].current_element = ack.focus().element ();
+
+          thread_view->unread_check ();
+        }
       }
   }
 
@@ -920,6 +924,26 @@ namespace Astroid {
     return DomUtils::assemble_data_uri (mime_type.c_str (), (const gchar *) data->get_data (), data->size ());
   } // }}}
 
+  void PageClient::scroll_to_bottom () {
+    AstroidMessages::Navigate n;
+    n.set_direction (AstroidMessages::Navigate_Direction_Down);
+    n.set_type (AstroidMessages::Navigate_Type_Extreme);
+
+    handle_ack (
+        AeProtocol::send_message_sync (AeProtocol::MessageTypes::Navigate, n, ostream, m_ostream, istream, m_istream)
+        );
+  }
+
+  void PageClient::scroll_to_top () {
+    AstroidMessages::Navigate n;
+    n.set_direction (AstroidMessages::Navigate_Direction_Up);
+    n.set_type (AstroidMessages::Navigate_Type_Extreme);
+
+    handle_ack (
+        AeProtocol::send_message_sync (AeProtocol::MessageTypes::Navigate, n, ostream, m_ostream, istream, m_istream)
+        );
+  }
+
   void PageClient::scroll_down_big () {
     AstroidMessages::Navigate n;
     n.set_direction (AstroidMessages::Navigate_Direction_Down);
